#version 430 core

layout(local_size_x = 16, local_size_y = 16) in;
layout(binding = 0, rgba8ui) uniform writeonly uimage2D outputImage;

uniform float time;
uniform int size;
uniform ivec2 origin;

const mat2 m = mat2(0.8, 0.6, -0.6, 0.8);

float rand(in vec2 n) {
    return fract(1E4 * sin(17.0 * n.x + n.y * 0.1) * (0.1 + abs(sin(n.y * 13.0 + n.x))));
}

float noise(vec2 p) {
    vec2 i = floor(p);
    vec2 f = fract(p);
    float a = rand(i);
    float b = rand(i + vec2(1.0, 0.0));
    float c = rand(i + vec2(0.0, 1.0));
    float d = rand(i + vec2(1.0, 1.0));
    vec2 u = f * f * (3.0 - 2.0 * f);
    return mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}

float fbm(vec2 p) {
    float f = 0.0;
    f += 0.5000 * noise(p);
    p = m * p * 2.02;
    f += 0.2500 * noise(p);
    p = m * p * 2.03;
    f += 0.1250 * noise(p);
    p = m * p * 2.01;
    f += 0.0625 * noise(p);
    p = m * p * 2.04;
    f /= 0.9375;
    return f;
}

float pattern(in vec2 p, out vec2 q, out vec2 r) {
    float t = time / 20.0;
    q.x = fbm(p + vec2(0.0, 0.1) + 2.0 * t * 0.22);
    q.y = fbm(p - vec2(0.0, 0.1) - 2.0 * t * 0.22);
    r.x = fbm(p + q * vec2(0.2, 0.1) + 2.0 * sin(t * 0.22));
    r.y = fbm(p - q * vec2(0.2, 0.1) - 2.0 * cos(t * 0.22));
    return fbm(p + 0.9 * r);
}

void main(void) {
    ivec2 offset = ivec2(gl_GlobalInvocationID.xy);
    vec2 uv = gl_GlobalInvocationID.xy / float(size);
    vec2 qo;
    vec2 ro;
    float f = pattern(8.0 * uv, qo, ro);
    vec3 col = vec3(0.0);
    col = mix(vec3(0.0), vec3(1.0), f * 1.2);
    col = mix(col, vec3(0.0), dot(qo, qo) * 1.5);
    col = mix(col, vec3(1.0), f / 0.82);
    uint outputColor = uint(col.r * 255.0);
    imageStore(outputImage, origin + offset, uvec4(uvec3(outputColor), 255));
}